#include "tester_adc_func.h"
#include "adc.h"

TESTER_ADCTypeDef tester_adc = {0};

uint16_t ADC_DMA_Buff[ADC_DMA_BUFF_SIZE];
uint16_t ADC_DMA_HalfBuff[ADC_DMA_BUFF_SIZE/2];

void ADC_DMA_Handler(TESTER_ADCTypeDef *adc)
{
  int flag_parse_dma = 0;
  int buff_start_ind = 0;
  int buff_end_ind = 0;    
  
  if(adc->f.dmaHandlerbusy)
    return;
  
  
  
  adc->f.dmaHandlerbusy = 1;    
  __disable_irq();
  
  if(DMA1->ISR & DMA_FLAG_HT1)
  {
    memcpy(ADC_DMA_HalfBuff, ADC_DMA_Buff, sizeof(ADC_DMA_HalfBuff));
    
    adc->f.dmaBufferHalfDone = 1;
    
    buff_start_ind = 0;
    buff_end_ind = ADC_BUFF_SIZE/2;
    flag_parse_dma = 1;
  }
  else if (DMA1->ISR & DMA_FLAG_TC1)
  {
    memcpy(ADC_DMA_HalfBuff, (ADC_DMA_Buff+ADC_DMA_BUFF_SIZE/2), sizeof(ADC_DMA_HalfBuff));
    
    
    adc->f.dmaBufferHalfDone = 2;
    
    buff_start_ind = ADC_BUFF_SIZE/2;
    buff_end_ind = ADC_BUFF_SIZE;
    flag_parse_dma = 1;
  }
  else
  {
    flag_parse_dma = 0;
    TrackerCnt_Warn(adc->adc_err);
  }  
  
  __enable_irq();
  if(adc->f.enable_dmaBuffTransfer && flag_parse_dma)
  {
    int dma_ind = 0;
    for(int buff_ind = buff_start_ind; 
        buff_ind < buff_end_ind; 
        buff_ind++)
    {
      
      
      adc->chAdc.ADC_Buff[buff_ind] = ADC_DMA_HalfBuff[dma_ind];
      
      dma_ind += 1;
    }
    
    printf_adc_it("\n%d: ADC DMA IT Done %d-half", uwTick, adc->f.dmaBufferHalfDone);
    adc->f.dmaHandlerbusy = 0;
    TrackerCnt_Ok(adc->adc_err);
  }
  else
  {
    TrackerCnt_Warn(adc->adc_err);
  }  
}


void TESTER_ADC_StructInit(TESTER_ADCTypeDef *adc)
{
  ClearStruct(*adc);
  adc->hadc = &hadc1;
  adc->chAdc.pulse_width = 4;
  adc->filter.array_size = FOSTER_STUDENT_N;
  adc->chAdc.ChMask = ADC_MEASURE_CHANNEL_0;
  
#ifdef FOSTER_STUDENT_USER_T_CRITICAL
  adc->filter.t_critical = FOSTER_STUDENT_USER_T_CRITICAL;
#else
  adc->filter.precise_table_ind = 2;
#endif
//  adc->DMA_HalfBuff = ADC_DMA_HalfBuff;
//  adc->DMA_Buff = ADC_DMA_Buff;
}


HAL_StatusTypeDef ADC_DMA_StartRead(TESTER_ADCTypeDef *adc)
{
  HAL_StatusTypeDef res;
  
  /* Очистка буферов каналов */
  ClearStruct(adc->chAdc.ADC_Buff);
  
  /* Очистка флага какая половина DMA уже готова */
  adc->f.dmaBufferHalfDone = 0; // никакая, данные DMA пока не готовы в принципе
  
  /* Разрешение заполнения буферов каналов */
  adc->f.enable_dmaBuffTransfer = 1;
  /* Выставления флага, что АЦП работает */
  adc->f.adc_running = 1;

  printf_adc("\n%d: ADC DMA Start", uwTick);

  /* Старт АЦП с DMA */
  res = HAL_ADC_Start_DMA(adc->hadc, (uint32_t *)ADC_DMA_Buff, ADC_DMA_BUFF_SIZE);
  
  /* Если старт неудачен - запрет заполнения буферов каналов */
  if(res != HAL_OK)
  {
    adc->f.enable_dmaBuffTransfer = 0;
    adc->f.adc_running = 0;
  }
  
  return res;
}

HAL_StatusTypeDef ADC_DMA_StopRead(TESTER_ADCTypeDef *adc)
{
  HAL_StatusTypeDef res;
  if(adc->f.adc_running)
  {
    /* Остановка АЦП */
    res = HAL_ADC_Stop_DMA(adc->hadc);
    
    /* Очищение флагов АЦП */
    adc->f.adc_running = 0;
    adc->f.enable_dmaBuffTransfer = 0;
    adc->f.dmaHandlerbusy = 0;
   

    printf_adc("\n%d: ADC DMA Stop", uwTick);
  }
  
  return res;
}

void ADC_DMA_Fuel_Cell_Reset(TESTER_ADCChannelTypeDef *adc_fc)
{
  adc_fc->ADC_Filtered = 0;
  adc_fc->ADC_Max = 0;
  adc_fc->ADC_Min = 0;
  adc_fc->max_value_ind = 0;
  adc_fc->state = 0;
}

void ADC_DMA_Channels_Prepare(TESTER_ADCChannelTypeDef *adc_fuel_cells, uint8_t MaskForChannels, uint8_t state)
{
  for(int i = 0; i < 4; i++)
  {
    if(MaskForChannels&(1<<i))
    {
      ADC_DMA_Fuel_Cell_Reset(&adc_fuel_cells[i]);
      adc_fuel_cells[i].state = state;
    }
  }
}

//uint8_t FosterStudent(TESTER_ADCChannelTypeDef *adc_fc, TESTER_ADCFilterTypeDef *Filter, uint32_t ind)
//{
//#ifndef FOSTER_STUDENT_USER_T_CRITICAL
//  uint32_t t_critical[] = FOSTER_STUDENT_T_CRITICALS;
//#endif //FOSTER_STUDENT_USER_T_CRITICAL
//  uint8_t res = 0;
//  
//  uint8_t m = 0;
//  uint8_t l = 0;
//  
//  uint16_t max = adc_fc->ADC_Buff[ind];  
//  uint16_t min = adc_fc->ADC_Buff[ind]; 
//  
//  int16_t d_sum = 0;  
//  uint32_t t;  
//  
//  uint16_t arr_size;
//  if(ind < Filter->array_size)
//    return 0xFF;
//    
//  for(int i = ind; i > ind - Filter->array_size; i--)
//  {
//      
//    if(adc_fc->ADC_Buff[i] > max)
//    {
//      max = adc_fc->ADC_Buff[i];
//      m = 1;
//    }
//    else
//      m = 0;
//    if(adc_fc->ADC_Buff[i] < min)
//    {
//      min = adc_fc->ADC_Buff[i];
//      l = 1;
//    }
//    else
//      l = 0;
//    d_sum += (m-l);
//  }
//  
//#define fixed_base 1000
//  if(d_sum < 0)
//    d_sum = -d_sum;
//  t = d_sum*fixed_base*fixed_base/FOSTER_STUDENT_MSE;
//
//  Filter->t_current = t;  
//#ifndef FOSTER_STUDENT_USER_T_CRITICAL  
//  Filter->t_critical = t_critical[Filter->precise_table_ind];
//#endif //FOSTER_STUDENT_USER_T_CRITICAL
//  
//  
//  if(Filter->t_current > Filter->t_critical)
//  {
//    res = 1;
//  }
//  else
//  {
//    res = 0;
//  }
//  
//  
//  
//  return res;
//}


void ADC_DMA_WaitForStableVoltage(TESTER_ADCChannelTypeDef *adc_fc, TESTER_ADCFilterTypeDef *Filter, uint32_t ind)
{
//  if(FosterStudent(adc_fc, Filter, ind) == 0)
//  {
//    /* Сохраняем индекс еще стабильного напряжения */
//    adc_fc->stable_start_ind = ind - Filter->array_size/2;
//    /* Переключение на стабильное напряжение найдено */
//    adc_fc->state = WAIT_FOR_DROP;
//  }
}


void ADC_DMA_WaitForDropVoltage(TESTER_ADCChannelTypeDef *adc_fc, TESTER_ADCFilterTypeDef *Filter, uint32_t ind)
{  
//  if(FosterStudent(adc_fc, Filter, ind) == 1)
//  {
//    /* Сохраняем индекс еще стабильного напряжения */
//    adc_fc->stable_end_ind = ind-Filter->array_size/2;
//    /* Переключение на стабильное напряжение найдено */
//    adc_fc->state = STABLE_FOUND;
//  }
}

void ADC_DMA_CalcStabilized(TESTER_ADCChannelTypeDef *adc_fc)
{
  uint16_t tmp_val = 0;
  uint32_t val_sum = 0;  
  
  adc_fc->ADC_Max = 0;
  adc_fc->ADC_Min = 0xFFFF;
  
  /* Просмотр стабильных значений в буфере */
  for(int i = 0; i < ADC_BUFF_SIZE; i++)
  {    
    tmp_val = adc_fc->ADC_Buff[i];
    /* Обновление максимального значения */
    if(tmp_val > adc_fc->ADC_Max)
    {
      adc_fc->ADC_Max = tmp_val;
      adc_fc->max_value_ind = i;
    }
    
    /* Обновление минимального значения */
    if(tmp_val < adc_fc->ADC_Min)
    {
      adc_fc->ADC_Min = tmp_val;
    }
    
  }
  
  
  /* Расчет среднего значения в пике */
  for(int i = (adc_fc->max_value_ind - adc_fc->pulse_width)/2; i < (adc_fc->max_value_ind + adc_fc->pulse_width/2); i++)
  {
    tmp_val = adc_fc->ADC_Buff[i];
    /* Накопление значений для расчета среднего */
    val_sum += tmp_val;
  }  
  adc_fc->ADC_Filtered = val_sum/(adc_fc->pulse_width);
  
}

void ADC_DMA_ProcessSingleBuffer(TESTER_ADCChannelTypeDef *adc_fc)
{
  
}


void ADC_DMA_ProcessBuffer(TESTER_ADCTypeDef *adc)
{  
  ADC_DMA_ProcessSingleBuffer(&adc->chAdc);
}

void ADC_DMA_BufferHandler(TESTER_ADCChannelTypeDef *adc_fc, TESTER_ADCFilterTypeDef *Filter, uint32_t ind)
{
//  switch(adc_fc->state)
//  {
//    /* Канал не проверяется */
//    case NOT_CHECKING:
//      break;    
//    
//    /* Пока на каналах не зарядится кондер */
//    case WAIT_FOR_STABLE: 
////      ADC_DMA_WaitForStableVoltage(adc_fc, Filter, ind);
//      break;
//    
//    /* Проверка пока на первом канале не начнет разряжаться кондер */
//    case WAIT_FOR_DROP:  
////      ADC_DMA_WaitForDropVoltage(adc_fc, Filter, ind);
//      break;
//    
//    /* Стабильное напряжение найдено */
//    case STABLE_FOUND:
//    /* Стабильное напряжение НЕ найдено */
//    case STABLE_NOT_FOUND:
//    /* Конец стабильного напряжения НЕ найден */
//    case DROP_NOT_FOUND:
//      break;
//    
//    
//    /* Если состояние неопределенное */
//    default: 
//      adc_fc->state = NOT_CHECKING;
//      break;
//      
//  }
}


/**
  * @brief    Считывать АЦП до тех пор, пока буфер DMA не заполнится и не будет обработан
  */
HAL_StatusTypeDef ADC_DMA_ReadWhileDMA(TESTER_ADCTypeDef *adc, uint32_t Timeout)
{
  uint32_t tickstart = HAL_GetTick();
  
  /* Проверка принятых параметров функции */
  if(adc == 0)
  {
    ADC_DMA_StopRead(adc);
    
    printf_adc_err("\n%d: Error ADC: Handle is NULL)", uwTick);
    return HAL_ERROR;
  }
  if(Timeout == 0)
  {
    ADC_DMA_StopRead(adc);
    
    printf_adc_err("\n%d: Error ADC: Read timeout is NULL)", uwTick);
    return HAL_ERROR;
  }
  

  int adc_buff_ind = 1;
  uint16_t filter_halfend = ADC_BUFF_SIZE/2;
  uint16_t filter_end = ADC_BUFF_SIZE;
  uint8_t run_adc_check = 0;
  uint8_t expected_dmaBufferHalf = 1;
  
  /* Обработка АЦП */
  while(1)
  {
    /* Проверка на таймаут */
    if(HAL_GetTick() - tickstart > Timeout)
    {
      ADC_DMA_StopRead(adc);

      printf_adc_err("\n%d: Error ADC: Stable Not Found (Timeout)", uwTick);
      return HAL_TIMEOUT;
    }
    
    
    /* Запуск обработки данных из DMA буфера, когда они готовы */
    if( (expected_dmaBufferHalf == adc->f.dmaBufferHalfDone) && // Ожидаемая половина DMA буфера готова
        (run_adc_check == 0))                                 // Обработка предыдущей половины не активна
    {
      run_adc_check = 1;

      if(adc->f.dmaBufferHalfDone == 1)
        printf_adc_processing("\n%d: ADC Processing DMA start...", uwTick);
      else if(adc->f.dmaBufferHalfDone == 2)
        printf_adc_processing("\n%d: ADC Processing DMA 2-half start...", uwTick);

    }
    
    
    /* Обработка буфера DMA */
    if(run_adc_check)
    { 
      /* Вызов обработчиков буфера DMA для каждого канала, если он выбран */
      ADC_DMA_BufferHandler(&adc->chAdc, &adc->filter, adc_buff_ind);
      
      
      
      /* Переход на следующие во времени значения каналов АЦП */
      adc_buff_ind++;      
      /* если это первая половина DMA буфера */
      if(expected_dmaBufferHalf == 1) 
      {
        if(adc_buff_ind > filter_halfend)   // Проверка первой половины закончена
        {
          adc_buff_ind = filter_halfend;    // Выставляем индекс на вторую половину
          run_adc_check = 0;                // Сбрасываем флаг проверки каналов
          expected_dmaBufferHalf = 2;       // Устанавливаем ожидание готовности второй половины данных DMA буфера

          printf_adc_processing("\n%d: ADC Processing 1-half DMA complete, wait for 2-half...", uwTick);

        }
      }
      /* если это вторая половина DMA буфера */
      else if(expected_dmaBufferHalf == 2)
      {
        /* Если АЦП работает - останавливаем его */
        // note: пока без кольцевого режима, если за один цикл буфера ничего не удалось словить, вряд ли далее что-то появиться??
        if(adc->f.adc_running)
        {
          ADC_DMA_StopRead(adc);
          printf_adc_processing(": wait for processing complete...");
        }
        
        if(adc_buff_ind > filter_end)       // Проверка первой половины закончена
        {
          adc_buff_ind = 0;                 // Выставляем индекс на первую половину
          run_adc_check = 0;                // Сбрасываем флаг проверки каналов
          expected_dmaBufferHalf = 1;       // Устанавливаем ожидание готовности первой половины данных DMA буфера

          printf_adc_processing("\n%d: ADC Processing DMA complete", uwTick);
          break;
        }
      }
      /* если это "никакая" половина DMA буфера */
      else
      { /* Сброс всего */

        printf_adc_err("\n%d: ADC Processing DMA unexcpected half", uwTick);

        adc_buff_ind = 0;
        run_adc_check = 0;
        expected_dmaBufferHalf = 1;
      }
    }
  }
  
  ADC_DMA_CalcStabilized(&adc->chAdc);
  
  return HAL_OK;
}