/**
**************************************************************************
* @file tester_func.h
* @brief Заголовочный файл для функций МЗКТ.
**************************************************************************
@details

*************************************************************************/

	
#ifndef _TESTER_ADC_FUNC_H_
#define _TESTER_ADC_FUNC_H_

#include "mylibs_include.h"

#define ADC_MEASURE_CHANNEL_0_Pos   0x00
#define ADC_MEASURE_CHANNEL_1_Pos   0x01
#define ADC_MEASURE_CHANNEL_2_Pos   0x02
#define ADC_MEASURE_CHANNEL_3_Pos   0x03

#define ADC_MEASURE_CHANNEL_0   (0x01 << ADC_MEASURE_CHANNEL_0_Pos)
#define ADC_MEASURE_CHANNEL_1   (0x01 << ADC_MEASURE_CHANNEL_1_Pos)
#define ADC_MEASURE_CHANNEL_2   (0x01 << ADC_MEASURE_CHANNEL_2_Pos)
#define ADC_MEASURE_CHANNEL_3   (0x01 << ADC_MEASURE_CHANNEL_3_Pos)
#define ADC_CH_IGNORE           NULL // должно быть NULL!!!

#define Measure_Get_Channel(_chPtr_)          (_chPtr_)->Tadc->chAdc
#define Measure_Get_Voltage(_chPtr_)          ((_chPtr_)->Tadc->chAdc->U_Current)
#define Measure_Get_Zero(_chPtr_)             ((_chPtr_)->Tadc->chAdc->refADC_zero)
#define Measure_Get_Calibr(_chPtr_)           ((_chPtr_)->Tadc->chAdc->refADC_calibr)
#define Measure_Get_VoltStep(_chPtr_)         ((_chPtr_)->Tadc->chAdc->refU_step)
#define Measure_Get_Filtered(_chPtr_)         ((_chPtr_)->Tadc->chAdc->ADC_Filtered)

#define Measure_Calc_CalibrDiff(_chPtr_)      abs((int)Measure_Get_Calibr(_chPtr_) - Measure_Get_Zero(_chPtr_))
#define Measure_Calc_FilterDiff(_chPtr_)      abs((int)Measure_Get_Filtered(_chPtr_) - Measure_Get_Zero(_chPtr_))  
#define Measure_Calc_Voltage(_chPtr_)         ((float)Measure_Calc_FilterDiff(_chPtr_)*Measure_Get_VoltStep(_chPtr_))
  
//#define ADC_GetChannel(_AdcChPtr_)           (_AdcChPtr_)->Tadc->chAdc
////#define Measure_Get_Voltage(_AdcChPtr_)           ((_AdcChPtr_)->Tadc->chAdc->U_Current)
////#define Measure_Get_Zero(_AdcChPtr_)              ((_AdcChPtr_)->Tadc->chAdc->refADC_zero)
////#define Measure_Get_Calibr(_AdcChPtr_)            ((_AdcChPtr_)->Tadc->chAdc->refADC_calibr)
////#define Measure_Get_VoltStep(_AdcChPtr_)          ((_AdcChPtr_)->Tadc->chAdc->refU_step)
////#define Measure_Get_Filtered(_AdcChPtr_)          ((_AdcChPtr_)->Tadc->chAdc->ADC_Filtered)

//#define Channel_Calc_Diff(_AdcChPtr_, _value_)    ((int)(_value_) - (_AdcChPtr_)->refADC_zero)
//#define Channel_Calc_CalibrValue(_AdcChPtr_)      (abs(Channel_Calc_Diff((_AdcChPtr_), (_AdcChPtr_)->refADC_calibr)) )
//#define Channel_Calc_AdcValue(_AdcChPtr_)         (abs(Channel_Calc_Diff((_AdcChPtr_), (_AdcChPtr_)->ADC_Filtered)) )
//#define Channel_Calc_Voltage(_AdcChPtr_)          ((float)Channel_Calc_AdcValue(_AdcChPtr_)*(_AdcChPtr_)->refU_step)

typedef uint16_t ADC_BuffTypeDef[ADC_BUFF_SIZE];

typedef enum
{
  ADC_DISABLED              = 0,
  WAIT_FOR_FIRST_HALF       = 1,
  PROCESSING_FIRST_HALF     = 2,
  WAIT_FOR_SECOND_HALF      = 3,
  PROCESSING_SECOND_HALF    = 4,
  ADC_DONE                  = 5,
}ADC_ChannelState;


typedef struct
{  
  ADC_BuffTypeDef ADC_Buff;
  uint16_t        ADC_Filtered;
  float           U_Current;
  
  
  
  
  /* Parameter for calc voltage */
  uint16_t        refADC_zero;
  uint16_t        refADC_calibr;
  float           refU_step;
  
  uint16_t        ADC_Max;
  uint16_t        ADC_Min;
   
  unsigned        pulse_width; 
  unsigned        max_value_ind; 
  
  uint8_t         ChMask;
  ADC_ChannelState state;
  
  ADCChannel_TrackerTypeDef   adc_ch_err;
}TESTER_ADCChannelTypeDef;

typedef struct
{
  int32_t t_current;
  int32_t t_critical;
  
#ifdef FOSTER_STUDENT_USER_T_CRITICAL
#else
  uint8_t precise_table_ind;  
#endif
  uint16_t array_size;
  
}TESTER_ADCFilterTypeDef;


typedef struct
{
  TESTER_ADCChannelTypeDef chAdc;
  
  ADC_HandleTypeDef *hadc; 

  TESTER_ADCFilterTypeDef filter;
  
  struct
  {
    
    unsigned adc_running:1;
    unsigned enable_dmaBuffTransfer:1;
    unsigned dmaBufferHalfDone:2;
    unsigned dmaHandlerbusy:1;
  }f;
  ADC_TrackerTypeDef adc_err;
}TESTER_ADCTypeDef;


extern TESTER_ADCTypeDef tester_adc;
extern uint16_t ADC_DMA_Buff[ADC_DMA_BUFF_SIZE];
extern uint16_t ADC_DMA_HalfBuff[ADC_DMA_BUFF_SIZE/2];

void TESTER_ADC_StructInit(TESTER_ADCTypeDef *adc);

void ADC_DMA_Handler(TESTER_ADCTypeDef *adc);

void ADC_DMA_ProcessBuffer(TESTER_ADCTypeDef *adc);
void ADC_DMA_Channels_Prepare(TESTER_ADCChannelTypeDef *adc_fuel_cells, uint8_t MaskForChannels, uint8_t state);
HAL_StatusTypeDef ADC_DMA_StartRead(TESTER_ADCTypeDef *adc);
HAL_StatusTypeDef ADC_DMA_ReadWhileDMA(TESTER_ADCTypeDef *adc, uint32_t Timeout);


#ifndef printf_adc_result
#define printf_adc_result(...)
#endif
#ifndef printf_adc_processing
#define printf_adc_processing(...)
#endif

#ifndef printf_adc_err
#define printf_adc_err(...)
#endif

#ifndef printf_adc_it
#define printf_adc_it(...)
#endif

#ifndef printf_adc
#define printf_adc(...)
#endif
#endif //_TESTER_ADC_FUNC_H_

