/**
**************************************************************************
* @file tester_config.h
* @brief Конфигурации для тестера диодов
**************************************************************************
* @defgroup    TESTER_CONFIGS    Configs for project
* @brief       Конфигурации для тестера диодов
* @details     
@{
*************************************************************************/
#ifndef _TESTER_CONFIG_H_
#define _TESTER_CONFIG_H_


/** 
	* @addtogroup TESTER_ADC_CONFIG   Configs for ADC
	* @ingroup 		TESTER_CONFIGS
  * @brief      Конфигурации для АЦП
	@{
	*/
  
#define ADC_BUFF_SIZE                           (500)               ///< Размер буфера АЦП
#define ADC_DMA_BUFF_SIZE                       (ADC_BUFF_SIZE*4)   ///< Размер буфера ДМА (4 канала)

#define ADC_U_CALIBR                            ((float)1.1)        ///< Напряжение, которое подается при калибровке
  
#define ADC_READ_TIMEOUT_MS                     20    ///< Таймаут на ожидание разрядки конденсатора
#define ADC_ZERO_WAIT_TIMEOUS_MS                100   ///< Таймаут на ожидание нуля (при калибровке)

#define FOSTER_STUDENT_N                        40    ///< Выборка для определения начала разрядки конденсатора
#define FOSTER_STUDENT_MSE                      2561  /*!<  @brief Среднеквадратическая ошибка для выборки размером FOSTER_STUDENT_N, умноженная на 1000
                                                            @details Выражение для СКО: sqrt(2*ln(N) - 0,8456) 
                                                            URL: https://old.stgau.ru/company/personal/user/7750/files/lib/%D0%A1%D1%82%D0%B0%D1%82%D0%B8%D1%81%D1%82%D0%B8%D1%87%D0%B5%D1%81%D0%BA%D0%B8%D0%B5%20%D0%BC%D0%B5%D1%82%D0%BE%D0%B4%D1%8B%20%D0%BF%D1%80%D0%BE%D0%B3%D0%BD%D0%BE%D0%B7%D0%B8%D1%80%D0%BE%D0%B2%D0%B0%D0%BD%D0%B8%D1%8F/%D0%9F%D1%80%D0%B0%D0%BA%D1%82%D0%B8%D0%BA%D1%83%D0%BC_%D1%82%D0%B5%D0%BC%D0%B0%201.pdf
                                                            */

/**
  * @brief    Критическое значение Стьюдента
  * @details  Используется для определения разрядки конденсатора.
  *           Записывается в единицах*1000
  * @note     Если закомментировать, то будет использоваться таблица указанная в @ref FOSTER_STUDENT_T_CRITICALS
              с индексом, указанным в @ref precise_table_ind
  */
#define FOSTER_STUDENT_USER_T_CRITICAL          1600//2650//2500
/**
  * @brief    Критическое значение Стьюдента для калибровки
  * @details  Используется для определения разрядки конденсатора при калибровке.
  *           Записывается в единицах*1000
  * @note     Если закомментировать, то будет использоваться значение указанное в @ref FOSTER_STUDENT_USER_T_CRITICAL
  */
//#define FOSTER_STUDENT_USER_T_CRITICAL_CALIBR   2600//2650//2500
/**
  * @brief    Таблица критических значений Стьюдента с разной погрешностью
  * @details  Используется для определения разрядки конденсатора при закомментированном
  *           @ref FOSTER_STUDENT_USER_T_CRITICAL
  */
#define FOSTER_STUDENT_T_CRITICALS              {257, 688, 1328, 1729, 2093, 2539}
/** TESTER_ADC_CONFIG
  * @}
  */


/** TESTER_CONFIGS
  * @}
  */

#endif //_TESTER_CONFIG_H_

